package com.io.yaohun.worldbosshurt.listener;

import com.io.yaohun.worldbosshurt.Main;
import com.io.yaohun.worldbosshurt.data.BossData;
import com.io.yaohun.worldbosshurt.data.RewardData;
import com.io.yaohun.worldbosshurt.event.WorldBossDamageEvent;
import com.io.yaohun.worldbosshurt.event.WorldBossDeathEvent;
import com.io.yaohun.worldbosshurt.mangage.BossManage;
import com.io.yaohun.worldbosshurt.util.BossStack;
import com.io.yaohun.worldbosshurt.util.BossUtil;
import io.lumine.xikage.mythicmobs.api.bukkit.events.MythicMobDeathEvent;
import me.Demon.DemonPlugin.DemonAPI;
import org.bukkit.Bukkit;
import org.bukkit.entity.Entity;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.scheduler.BukkitRunnable;

import java.text.DecimalFormat;
import java.util.*;
import java.util.stream.Collectors;

public class BossKillsDeath implements Listener {

    @EventHandler
    public void onkilldrop(MythicMobDeathEvent e){
        if(e.getKiller() instanceof Player) { // 判断斩杀者是玩家
            Entity entity = e.getEntity();
            BossManage bossManage = Main.bossManage;
            String customName = entity.getCustomName();
            if(customName != null && bossManage.isCustomNameContains(customName)){
                BossData bossData = bossManage.getNameToData(customName);
                if(bossData == null){return;}
                String worldName = entity.getWorld().getName();
                if (!worldName.equalsIgnoreCase("teamDungee")) {
                    Player player = ((Player) e.getKiller()).getPlayer();
                    String message = BossUtil.getFinallyKills().replace("%p%", player.getName()); // 获取最终击杀提示
                    Bukkit.broadcastMessage(message.replace("%mobs%",bossData.getBossName())); // 发送最终击杀提示
                    BossUtil.addMonthlyKillBoss(player.getName()); // 为玩家累积最终斩杀次数
                    String Kills_Group_Key = bossData.getKillsReward(); // 获取击杀奖励组名
                    RewardData killsRewardData = Main.rewardManage.getRewardData(Kills_Group_Key);
                    // 执行命令并发放奖励
                    if(bossData.getKillsReward() != null){
                        if (!Kills_Group_Key.equalsIgnoreCase("魂环")) {
                            List<String> commands = killsRewardData.getKillsCmdList();
                            DemonAPI.ConsoleOutCommandList(player, commands);
                        }else{
                            sendRingRewards(player,bossData,killsRewardData);
                        }
                    }
                    new BukkitRunnable() {
                        public void run() {
                            statisticalSettlementOfDamages(bossData,Main.rewardManage.getRewardData(bossData.getTopReward()));
                            cancel();
                        }
                    }.runTaskLater(Main.plugin, 20L);
                }
            }
        }
    }

    /*
    * 将对boss造成伤害的存进Hash
    * 并进行排序随后发送全服公告
    * */
    public static void statisticalSettlementOfDamages(BossData bossData,RewardData rewardData) {
        double totalDamage = 0; // 获取本次造成总伤害
        HashMap<String, Double> hashMap = new HashMap<>();
        Map<Player, Double> damageMap = BossDamageByPlayer.damageMap;
        List<Player> playerList = new ArrayList<>();
        for (Player player : damageMap.keySet()) {
            if (player != null && player.isOnline()) {
                double damage = damageMap.get(player); // 获取该玩家造成伤害
                totalDamage += damage; // 统计玩家造成总计伤害
                hashMap.put(player.getName(), damage); // 将玩家名和伤害存储
                playerList.add(player); // 将玩家存入在线玩家列表
                WorldBossDamageEvent worldBossDamageEvent = new WorldBossDamageEvent(player,bossData,damage);
                Bukkit.getPluginManager().callEvent(worldBossDamageEvent);
            }
        }
        if (hashMap.size() < 1) { return; }
        LinkedHashMap<String, Double> sortedMap = hashMap.entrySet()
                .stream()
                .sorted(Map.Entry.comparingByValue(Comparator.reverseOrder()))
                .collect(Collectors.toMap(
                        Map.Entry::getKey,
                        Map.Entry::getValue,
                        (e1, e2) -> e1,
                        LinkedHashMap::new
                ));
        Bukkit.broadcastMessage("§c§M    §6§m    §e§m    §a§m    §b§m    §C§L▍ §c§l伤§6§l害§E§L统§A§l计§B§L表 §C§l▍§b§M    §a§M    §e§M    §6§m    §c§m    §r");
        Bukkit.broadcastMessage("§e世界级Boss: "+bossData.getBossName()+"  §e伤害排名如下:");
        int rank = 1;
        for (String top_Name : sortedMap.keySet()){
            if(rank == 6){break;}
            double damage = sortedMap.get(top_Name);
            Bukkit.broadcastMessage("§e第" + rank + "名 - §c" + top_Name + " §e总计伤害: §c" + new DecimalFormat("00.00").format(damage) + "k §e伤害占总比: §c" + new DecimalFormat("0.00").format(damage / totalDamage * 100) + "%");
            rank++;
        }
        Bukkit.broadcastMessage("§c§M    §6§m    §e§m    §a§m    §b§m    §C§L▍ §c§l伤§6§l害§E§L统§A§l计§B§L表 §C§l▍§b§M    §a§M    §e§M    §6§m    §c§m    §r");
        for (Player player : playerList){
            if(BossDamageByPlayer.damageMap.get(player) != null) {
                double damage = BossDamageByPlayer.damageMap.get(player); // 获取玩家造成伤害
                player.sendMessage("§e您造成的总伤害: §c" + new DecimalFormat(",###.00").format(damage) + "k  §e伤害占总比: §c" + new DecimalFormat("0.00").format(damage / totalDamage * 100) + "%");
                if(rewardData != null) {
                    // 判断玩家造成伤害是否达到100K并发放奖励
                    if (damage >= 10) {
                        List<String> damageRewardList = rewardData.getDamageCmdList();
                        DemonAPI.ConsoleOutCommandList(player, damageRewardList);
                    } else {
                        DemonAPI.sendMessage(player, "你造成的总伤害不足§4100,000§a无法获得参与奖。");
                    }
                }
            }
        }
        WorldBossDeathEvent worldBossDeathEvent = new WorldBossDeathEvent(bossData,rewardData,sortedMap);
        Bukkit.getPluginManager().callEvent(worldBossDeathEvent);
    }

    public void sendRingRewards(Player player,BossData bossData,RewardData rewardData) {
        String name = player.getName();
        if(DemonAPI.getRandomInt(100,1) >= 10){
            DemonAPI.sendMessage(player,"很遗憾,这只灵兽的最终一击并未掉落任何奖励。");
        }else{
            String bossKey = bossData.getBossKey();
            if (DemonAPI.getRandomInt(100,1) >= 2) {
                if (DemonAPI.getRandomInt(100, 1) > 10) {
                    player.getInventory().addItem(BossStack.BossSlashProof(bossKey));
                    Bukkit.broadcastMessage("§6[§4公告§6] §e"+name+" §a通过最终一击获得了 §c§l猎杀凭证 §f("+bossKey+")");
                } else {
                    if(rewardData.getRewardKey().contains("百万年")){
                        Bukkit.dispatchCommand(Bukkit.getConsoleSender(), "mythicmobs items give " + name + " 百万年" + bossKey + "灵环无献祭");
                        DemonAPI.sendMessage(player,"恭喜你获得了 §6§l百万年灵环 §f(" + bossKey + ")");
                        Bukkit.broadcastMessage("§6[§4公告§6] §e" + name + " §a通过最终一击获得了 §6§l百万年灵环 §f(" + bossKey + ")");
                    } else {
                        Bukkit.dispatchCommand(Bukkit.getConsoleSender(), "mythicmobs items give " + name + " 十万年" + bossKey + "灵环无献祭");
                        DemonAPI.sendMessage(player,"恭喜你获得了 §c§l十万年灵环 §f(" + bossKey + ")");
                        Bukkit.broadcastMessage("§6[§4公告§6] §e" + name + " §a通过最终一击获得了 §c§l十万年灵环 §f(" + bossKey + ")");
                    }
                }
            } else {
                if(rewardData.getRewardKey().contains("百万年")){
                    Bukkit.dispatchCommand(Bukkit.getConsoleSender(), "mythicmobs items give " + name + " 百万年" + bossKey + "灵环");
                    DemonAPI.sendMessage(player,"恭喜你获得了 §6§l百万年灵环 §f(" + bossKey + ")§d[含献祭]");
                    Bukkit.broadcastMessage("§6[§4公告§6] §e" + name + " §a通过最终一击获得了 §6§l百万年灵环 §f(" + bossKey + ")§d[含献祭]");
                }else {
                    Bukkit.dispatchCommand(Bukkit.getConsoleSender(), "mythicmobs items give " + name + " 十万年" + bossKey + "灵环");
                    DemonAPI.sendMessage(player,"恭喜你获得了 §c§l十万年灵环 §f(" + bossKey + ")§d[含献祭]");
                    Bukkit.broadcastMessage("§6[§4公告§6] §e" + name + " §a通过最终一击获得了 §c§l十万年灵环 §f(" + bossKey + ")§d[含献祭]");
                }
            }
        }
    }

}
